package test.client;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;

import test.MyWsOtherSigFirst;
import test.MyWsOtherSigFirstBean;
import test.MyWsOtherSigSecond;
import test.MyWsOtherSigSecondBean;
import test.MyWsSameSigFirst;
import test.MyWsSameSigFirstBean;
import test.MyWsSameSigSec;
import test.MyWsSameSigSecBean;

public class Main {
   private final String urlBase;

   private Main(String urlBase) {
      this.urlBase = urlBase;
   }

   private void callMyWsSameSigFirst(final long value) {
      final String wsName = "MyWsSameSigFirst";
      try {
         final Service service = createService(wsName);

         final MyWsSameSigFirst myWs = (MyWsSameSigFirst) service.getPort(MyWsSameSigFirst.class);
         final long returnValue = myWs.callMe(value);
         System.out.println("Success call " + wsName);

         final MyWsSameSigFirstBean myWsBean = new MyWsSameSigFirstBean();
         final long expectedValue = myWsBean.callMe(value);
         if (expectedValue == returnValue) {
            System.out.println("Return value is correct: " + returnValue);
         } else {
            System.out.println("Ws call " + wsName + " returns " + returnValue + " but expected " + expectedValue);
         }
      } catch (Exception e) {
         System.err.println("Failed to call " + wsName);
         e.printStackTrace();
      }
   }

   private void callMyWsSameSigSec(final long value) {
      final String wsName = "MyWsSameSigSec";
      try {
         final Service service = createService(wsName);

         final MyWsSameSigSec myWs = (MyWsSameSigSec) service.getPort(MyWsSameSigSec.class);
         final long returnValue = myWs.callMe(value);
         System.out.println("Success call " + wsName);

         final MyWsSameSigSecBean myWsBean = new MyWsSameSigSecBean();
         final long expectedValue = myWsBean.callMe(value);
         if (expectedValue == returnValue) {
            System.out.println("Return value is correct: " + returnValue);
         } else {
            System.out.println("Ws call " + wsName + " returns " + returnValue + " but expected " + expectedValue);
         }
      } catch (Exception e) {
         System.err.println("Failed to call " + wsName);
         e.printStackTrace();
      }
   }

   private void callMyWsOtherSigFirst(final String value) {
      final String wsName = "MyWsOtherSigFirst";
      try {
         final Service service = createService(wsName);

         final MyWsOtherSigFirst myWs = (MyWsOtherSigFirst) service.getPort(MyWsOtherSigFirst.class);
         final String returnValue = myWs.callOther(value);
         System.out.println("Success call " + wsName);

         final MyWsOtherSigFirstBean myWsBean = new MyWsOtherSigFirstBean();
         final String expectedValue = myWsBean.callOther(value);
         if (expectedValue.equals(returnValue)) {
            System.out.println("Return value is correct: " + returnValue);
         } else {
            System.out.println("Ws call " + wsName + " returns " + returnValue + " but expected " + expectedValue);
         }
      } catch (Exception e) {
         System.err.println("Failed to call " + wsName);
         e.printStackTrace();
      }
   }

   private void callMyWsOtherSigSecond(final boolean value) {
      final String wsName = "MyWsOtherSigSecond";
      try {
         final Service service = createService(wsName);

         final MyWsOtherSigSecond myWs = (MyWsOtherSigSecond) service.getPort(MyWsOtherSigSecond.class);
         final boolean returnValue = myWs.callOther(value);
         System.out.println("Success call " + wsName);

         final MyWsOtherSigSecondBean myWsBean = new MyWsOtherSigSecondBean();
         final boolean expectedValue = myWsBean.callOther(value);
         if (expectedValue == returnValue) {
            System.out.println("Return value is correct: " + returnValue);
         } else {
            System.out.println("Ws call " + wsName + " returns " + returnValue + " but expected " + expectedValue);
         }
      } catch (Exception e) {
         System.err.println("Failed to call " + wsName);
         e.printStackTrace();
      }
   }

   private Service createService(final String wsName) throws MalformedURLException {
      final String urlString = this.urlBase + "/" + wsName + "BeanService/" + wsName + "Bean?wsdl";
      final URL url = new URL(urlString);
      final QName qname = new QName("http://test/", wsName + "BeanService");
      final Service service = Service.create(url, qname);
      return service;
   }

   public static void main(String[] args) throws Exception {
      final Main main = new Main("http://localhost:8080");
      main.callMyWsSameSigFirst(10);
      main.callMyWsSameSigSec(10);
      main.callMyWsOtherSigFirst("Test");
      main.callMyWsOtherSigSecond(true);
   }
}
